/******************************************************************************/
/*
 * eRIC Example Advanced Encryption Standard(AES) V1.1
 * Low Power Radio Solutions
 * March 2016
 * Built with CCS V6.1.2.00015
 * This example will Encrypt,decrypt or do both on transmission and reception of data as per #defines below.
 * Data is sent whenever Pin19 is held high. Data is received and delivered to Uart on Pin3 at 19200 baud rate.
 * 16 Bytes AES key used in this example is ABCDEFGHIJKLMNOP.
 *
 */

/******************************************************************************/
//Version V1.1
#include<cc430f5137.h>
#include "eRIC.h"
#include <stdio.h>
#include <string.h>

//#define AES_Encrypt          //Data is encrypted before transmitting but recevied data is not decrypted
//#define AES_Decrypt            //Data is decrypted after receiving but no encryption is done on transmitted data
#define AES_Encrypt_Decrypt  //Data is encrypted before transmitting and decrypted after receiving

volatile const char Data[] = {'A','E','S',' ','E','X','A','M','P','L','E'};

int main(void)
{
	eRIC_WDT_Stop();                   //Stop watch dog timer, just in case
	eRIC_GlobalInterruptDisable();     //Global interrupts disabled

	eROS_Initialise(434000000);       //Set 434Mhz frequency
	eRIC_SetCpuFrequency(4500000);    //Set CPU frequency to 4.5Mhz
	eRIC_Power = 12;                  //Set power to 12dbm
	eRIC_RadioUpdate();

	eRIC_Rx_Enable();                 //Enable Receiver

	eRIC_GlobalInterruptEnable();

	Pin19_SetAsInput();
	Pin19_PullDownEnable();

	eRIC_UARTAInitialise(19200);      //Setup Uart
	Pin3_FunctionUartATxOUT();

	eRIC_RfDataReceivedInterruptEnable();//V1.1

	volatile char i =0;


    //Set the AES key irrespective of encryption or decryption, as key will be same for both
	eRIC_AES_Key[0] = 0;             //At first Set the AES key for encryption and can change mode before encrypting or decrypting data
	while(i<16)
	{
		eRIC_AES_Key[i+1] = 'A'+i;
		i++;
	}
	eRIC_AES_ChangeKey();


	while(1)//Loop continously
	{
		if(Pin19_Read())
		{
			while(Pin19_Read()); //Wait while Pin19 is held high
			#if defined (AES_Encrypt) || defined (AES_Encrypt_Decrypt)    //Send AES encryption
			if(eRIC_AES_Key[0] !=0)     //IF AES key is not set with encryption mode, set the key with encryption mode
			{
				eRIC_AES_Key[0] = 0;
				eRIC_AES_SetKey();  //No need to use eRIC_AES_ChangeKey(); again, as key is same, it only needs changing mode
			}
			i =0;
			eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = 11; //Send first byte of data as byte count, as we are sending only 11 bytes
			while(i<16)
			{
				if(i<11)    //As we are encrypting only 11 bytes of data, put rest of the bytes as 0 to make 16 bytes to encrypt
					eRIC_AES_Data[i] = Data[i];
				else
					eRIC_AES_Data[i] = 0;
				i++;
			}
			eRIC_AES_Run();//encrypt data
			i =0;
			while(i<16)
			{
				eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = eRIC_AES_Data[i];
				i++;
			}
			eRIC_RfSenddata();
            #else  //Send normal data without AES encryption
			i =0;
			while(i<11)
			{
				eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = Data[i];
				i++;
			}
			eRIC_RfSenddata();
            #endif
		}
	}
}//End of main

//Added in V1.1
void eRIC_RfDataReceivedInterrupt()   //V1.1 Add code here to deal with  available received data..THis is triggered when interrupt is enabled and a packet is received
{
	volatile char i =0;
	if(eRIC_Rxdata_available)// Receive
	{
		#if defined (AES_Decrypt) || defined (AES_Encrypt_Decrypt)    //Receive encrypted data to decrypt
		if(eRIC_AES_Key[0]!=1)
		{
			eRIC_AES_Key[0] = 1;
			eRIC_AES_SetKey();
		}
		volatile unsigned char j = 0;
		volatile char counttimeout =0;
		volatile unsigned char Bytesinpacket = eRIC_ReadRfByte();//Reads the byte count of the overair packet which is first byte of packet
		for(i=0;i<Bytesinpacket;i+=16)
		{
			j=0;
			while(j<16)
			{
				if(eRIC_Rxdata_available)
				{
					counttimeout =0;
					eRIC_AES_Data[j] = eRIC_ReadRfByte();   //Reads each byte upto 16 bytes
					j++;
				}
				else
				{
					counttimeout++;    //If there is no byte then it waits for 10 ms
					eRIC_Delay(10);
				}
				if(counttimeout==50) //If there is no data for 500ms break this complete
				{
					while(eRIC_Rxdata_available)
						eRIC_ReadRfByte();   //FLush all bytes
					j=16;
				}
			}
			if(counttimeout==50) //If there is no data for 500ms break this complete
			{
				break;
			}
			//else
			{
				eRIC_AES_Run();  //Decrypt packet of 16 bytes
				j =0;
				while(((i+j)<Bytesinpacket) && (j<16))
				{
					eRIC_UartASendByte(eRIC_AES_Data[j]);        //send decrypted packet to Uart
					while(eRIC_UartATxBufferIsBusy());
					j++;
				}
			}
		}
		#else
		while(eRIC_Rxdata_available)
		{
			eRIC_UartASendByte(eRIC_ReadRfByte());
			while(eRIC_UartATxBufferIsBusy());
		}
		#endif
		}
}



